Index: html/docs/objectdefinitions.html
===================================================================
RCS file: /cvsroot/nagios/nagios/html/docs/objectdefinitions.html,v
retrieving revision 1.19
diff -u -p -r1.19 objectdefinitions.html
--- html/docs/objectdefinitions.html	4 Nov 2008 22:28:40 -0000	1.19
+++ html/docs/objectdefinitions.html	19 Nov 2008 04:39:51 -0000
@@ -157,6 +157,7 @@ Note:  Directives in red are required, w
 <tr><td></td><td class="Optional">statusmap_image</td><td class="Optional"><i>image_file</i></td></tr>
 <tr><td></td><td class="Optional">2d_coords</td><td class="Optional"><i>x_coord,y_coord</i></td></tr>
 <tr><td></td><td class="Optional">3d_coords</td><td class="Optional"><i>x_coord,y_coord,z_coord</i></td></tr>
+<tr><td></td><td class="Optional">services</td><td class="Optional"><i>servicelist</i></td></tr>
 
 <tr><td>&nbsp;&nbsp;&nbsp;</td><td colspan=2 class="Definition">}</td></tr>
 </table>
@@ -452,6 +453,41 @@ This variable is used to define coordina
 This variable is used to define coordinates to use when drawing the host in the <a href="cgis.html#statuswrl_cgi">statuswrl</a> CGI.  Coordinates can be positive or negative real numbers.  The origin for drawing is (0.0,0.0,0.0).  For reference, the size of the host cubes drawn is 0.5 units on each side (text takes a little more space).  The coordinates you specify here are used as the center of the host cube.
 </td>
 </tr>
+<tr>
+<td valign="top"><strong>services</strong>:</td>
+<td>
+This directive can be used to automatically instantiate <a href="#service">service</a>
+definitions for this host based on the indicated service templates.
+Multiple service template names should be separated by commas.
+<br>
+This directive may be used as an alternative to (or in addition to) using <a href="#service">service</a> definitions for this host.
+<br>
+The following sequence:
+<br>
+<pre>
+    define host {
+	host_name		bogus
+	address			192.168.1.254
+	services		template-check-disk-sda
+	...
+    }
+</pre>
+
+is exactly equivalent to the following sequence:
+
+<pre>
+    define host {
+	host_name		bogus
+	address			192.168.1.254
+        ...
+    }
+    define service {
+	host_name		bogus
+	use			template-check-disk-sda
+    }
+</pre>
+</td>
+</tr>
 </table>
 
 
Index: xdata/xodtemplate.c
===================================================================
RCS file: /cvsroot/nagios/nagios/xdata/xodtemplate.c,v
retrieving revision 1.181
diff -u -p -r1.181 xodtemplate.c
--- xdata/xodtemplate.c	23 Jun 2008 21:43:22 -0000	1.181
+++ xdata/xodtemplate.c	19 Nov 2008 04:39:51 -0000
@@ -124,6 +124,13 @@ char *xodtemplate_precache_file=NULL;
 
 int presorted_objects=FALSE;
 
+/*
+ * List of service templates to be automatically instantiated for
+ * the current host. This is used while reading a 'define host {}'
+ * section.
+ */
+char *xodtemplate_host_auto_services=NULL;
+
 
 /******************************************************************/
 /************* TOP-LEVEL CONFIG DATA INPUT FUNCTION ***************/
@@ -843,6 +850,9 @@ int xodtemplate_begin_object_definition(
 	xodtemplate_serviceextinfo *new_serviceextinfo=NULL;
 	register int x=0;
 
+	/* Used only by "host", but doesn't hurt to always initialize */
+	my_free( xodtemplate_host_auto_services );
+	xodtemplate_host_auto_services=NULL;
 
 	if(!strcmp(input,"service"))
 		xodtemplate_current_object_type=XODTEMPLATE_SERVICE;
@@ -3665,6 +3675,14 @@ int xodtemplate_add_object_property(char
 			temp_host->retain_nonstatus_information=(atoi(value)>0)?TRUE:FALSE;
 			temp_host->have_retain_nonstatus_information=TRUE;
 		        }
+		else if(!strcmp(variable,"services")){
+			if(strcmp(value,XODTEMPLATE_NULL)){
+				my_free( xodtemplate_host_auto_services );
+				xodtemplate_host_auto_services = (char *)strdup(value);
+				if ( xodtemplate_host_auto_services == NULL )
+					result=ERROR;
+			        }
+		        }
 		else if(!strcmp(variable,"register"))
 			temp_host->register_object=(atoi(value)>0)?TRUE:FALSE;
 		else if(variable[0]=='_'){
@@ -4707,6 +4725,87 @@ int xodtemplate_add_object_property(char
 int xodtemplate_end_object_definition(int options){
 	int result=OK;
 
+	/* Special case: if we just finished a "host", auto-generate
+	 * any requested services as if we had also read:
+	 *
+	 *	define service {
+	 *	    host_name HOSTNAME
+	 *	    use TEMPLATE
+	 *	}
+	 *
+	 * for each service name in the "services" statement.
+	 */
+	if ( ( xodtemplate_current_object_type == XODTEMPLATE_HOST ) &&
+	     ( xodtemplate_host_auto_services != NULL ) ) {
+		xodtemplate_host *host = (xodtemplate_host *) xodtemplate_current_object;
+		char *servicelist = xodtemplate_host_auto_services;
+		char *service;
+
+		/* Make sure that this is a true, honest-to-god, definition and
+		 * not a template.
+		 * We can't deal with this directive from a template (FIXME)
+		 */
+		if ( host->register_object ) {
+
+			/* Finish the object */
+			xodtemplate_current_object=NULL;
+			xodtemplate_current_object_type=XODTEMPLATE_NONE;
+
+			while ( ( result == OK ) &&
+				( ( service = my_strsep( &servicelist, "," ) ) != NULL ) ) {
+
+				char *line;
+
+				strip( service );
+
+				if ( service[0] == '\x0' ) {
+					continue;
+					}
+
+				/* Start */
+				result = xodtemplate_begin_object_definition("service", options, xodtemplate_current_config_file, host->_start_line );
+				if ( result != OK ) {
+					break;
+					}
+
+				/* "host_name HOSTNAME" */
+				asprintf( &line, "host_name %s", host->host_name );
+				result = xodtemplate_add_object_property( line, options );
+				my_free( line );
+				if ( result != OK ) {
+					break;
+					}
+
+				/* "use SERVICE_TEMPLATE" */
+				asprintf( &line, "use %s", service );
+				result = xodtemplate_add_object_property( line, options );
+				my_free( line );
+				if ( result != OK ) {
+					break;
+					}
+
+				/* end the service */
+				result = xodtemplate_end_object_definition( options );
+				if ( result != OK ) {
+					break;
+					}
+				}
+			}
+
+#ifdef NSCORE
+		if ( result != OK ) {
+			logit(NSLOG_CONFIG_ERROR,TRUE,
+			      "Error: Could not add auto service definition '%s' for host '%s' in file '%s' on line %d.\n",
+			      service,
+			      host->host_name,
+			      xodtemplate_config_file_name( host->_config_file ),
+			      host->_start_line );
+			}
+#endif
+
+		my_free( xodtemplate_host_auto_services );
+		xodtemplate_host_auto_services = NULL;
+		}
 
 	xodtemplate_current_object=NULL;
 	xodtemplate_current_object_type=XODTEMPLATE_NONE;
